/* hashmap.vala
 *
 * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
 * Copyright (C) 1997-2000  GLib Team and others
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <gee/hashmap.h>

typedef struct _GeeHashMapNode GeeHashMapNode;

#define GEE_HASH_MAP_TYPE_KEY_SET (gee_hash_map_key_set_get_type ())
#define GEE_HASH_MAP_KEY_SET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySet))
#define GEE_HASH_MAP_KEY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))
#define GEE_HASH_MAP_IS_KEY_SET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_KEY_SET))
#define GEE_HASH_MAP_IS_KEY_SET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_KEY_SET))
#define GEE_HASH_MAP_KEY_SET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetClass))

typedef struct _GeeHashMapKeySet GeeHashMapKeySet;
typedef struct _GeeHashMapKeySetClass GeeHashMapKeySetClass;
typedef struct _GeeHashMapKeySetPrivate GeeHashMapKeySetPrivate;

#define GEE_HASH_MAP_TYPE_KEY_ITERATOR (gee_hash_map_key_iterator_get_type ())
#define GEE_HASH_MAP_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIterator))
#define GEE_HASH_MAP_KEY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))
#define GEE_HASH_MAP_IS_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR))
#define GEE_HASH_MAP_IS_KEY_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_KEY_ITERATOR))
#define GEE_HASH_MAP_KEY_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorClass))

typedef struct _GeeHashMapKeyIterator GeeHashMapKeyIterator;
typedef struct _GeeHashMapKeyIteratorClass GeeHashMapKeyIteratorClass;
typedef struct _GeeHashMapKeyIteratorPrivate GeeHashMapKeyIteratorPrivate;

#define GEE_HASH_MAP_TYPE_VALUE_COLLECTION (gee_hash_map_value_collection_get_type ())
#define GEE_HASH_MAP_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollection))
#define GEE_HASH_MAP_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))
#define GEE_HASH_MAP_IS_VALUE_COLLECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
#define GEE_HASH_MAP_IS_VALUE_COLLECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_COLLECTION))
#define GEE_HASH_MAP_VALUE_COLLECTION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionClass))

typedef struct _GeeHashMapValueCollection GeeHashMapValueCollection;
typedef struct _GeeHashMapValueCollectionClass GeeHashMapValueCollectionClass;
typedef struct _GeeHashMapValueCollectionPrivate GeeHashMapValueCollectionPrivate;

#define GEE_HASH_MAP_TYPE_VALUE_ITERATOR (gee_hash_map_value_iterator_get_type ())
#define GEE_HASH_MAP_VALUE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIterator))
#define GEE_HASH_MAP_VALUE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))
#define GEE_HASH_MAP_IS_VALUE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR))
#define GEE_HASH_MAP_IS_VALUE_ITERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GEE_HASH_MAP_TYPE_VALUE_ITERATOR))
#define GEE_HASH_MAP_VALUE_ITERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorClass))

typedef struct _GeeHashMapValueIterator GeeHashMapValueIterator;
typedef struct _GeeHashMapValueIteratorClass GeeHashMapValueIteratorClass;
typedef struct _GeeHashMapValueIteratorPrivate GeeHashMapValueIteratorPrivate;

struct _GeeHashMapNode {
	gpointer key;
	gpointer value;
	GeeHashMapNode* next;
	guint key_hash;
};

struct _GeeHashMapKeySet {
	GObject parent_instance;
	GeeHashMapKeySetPrivate * priv;
};

struct _GeeHashMapKeySetClass {
	GObjectClass parent_class;
};

struct _GeeHashMapKeyIterator {
	GObject parent_instance;
	GeeHashMapKeyIteratorPrivate * priv;
};

struct _GeeHashMapKeyIteratorClass {
	GObjectClass parent_class;
};

struct _GeeHashMapValueCollection {
	GObject parent_instance;
	GeeHashMapValueCollectionPrivate * priv;
};

struct _GeeHashMapValueCollectionClass {
	GObjectClass parent_class;
};

struct _GeeHashMapValueIterator {
	GObject parent_instance;
	GeeHashMapValueIteratorPrivate * priv;
};

struct _GeeHashMapValueIteratorClass {
	GObjectClass parent_class;
};



struct _GeeHashMapPrivate {
	gint _array_size;
	gint _nnodes;
	GeeHashMapNode** _nodes;
	gint _nodes_length1;
	gint _stamp;
	GHashFunc _key_hash_func;
	GEqualFunc _key_equal_func;
	GEqualFunc _value_equal_func;
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

#define GEE_HASH_MAP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_TYPE_HASH_MAP, GeeHashMapPrivate))
enum  {
	GEE_HASH_MAP_DUMMY_PROPERTY,
	GEE_HASH_MAP_SIZE,
	GEE_HASH_MAP_KEY_HASH_FUNC,
	GEE_HASH_MAP_KEY_EQUAL_FUNC,
	GEE_HASH_MAP_VALUE_EQUAL_FUNC,
	GEE_HASH_MAP_K_TYPE,
	GEE_HASH_MAP_K_DUP_FUNC,
	GEE_HASH_MAP_K_DESTROY_FUNC,
	GEE_HASH_MAP_V_TYPE,
	GEE_HASH_MAP_V_DUP_FUNC,
	GEE_HASH_MAP_V_DESTROY_FUNC
};
#define GEE_HASH_MAP_MIN_SIZE 11
#define GEE_HASH_MAP_MAX_SIZE 13845163
static GeeSet* gee_hash_map_real_get_keys (GeeMap* base);
static GeeCollection* gee_hash_map_real_get_values (GeeMap* base);
static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointer key);
static gboolean gee_hash_map_real_contains (GeeMap* base, gconstpointer key);
static gpointer gee_hash_map_real_get (GeeMap* base, gconstpointer key);
static void gee_hash_map_real_set (GeeMap* base, gconstpointer key, gconstpointer value);
static gboolean gee_hash_map_real_remove (GeeMap* base, gconstpointer key);
static void gee_hash_map_real_clear (GeeMap* base);
static void gee_hash_map_resize (GeeHashMap* self);
static GObject * gee_hash_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash);
static void gee_hash_map_node_instance_init (GeeHashMapNode * self);
static void gee_hash_map_node_free (GeeHashMapNode* self);
struct _GeeHashMapKeySetPrivate {
	GeeHashMap* _map;
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

#define GEE_HASH_MAP_KEY_SET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_KEY_SET, GeeHashMapKeySetPrivate))
enum  {
	GEE_HASH_MAP_KEY_SET_DUMMY_PROPERTY,
	GEE_HASH_MAP_KEY_SET_MAP,
	GEE_HASH_MAP_KEY_SET_SIZE,
	GEE_HASH_MAP_KEY_SET_K_TYPE,
	GEE_HASH_MAP_KEY_SET_K_DUP_FUNC,
	GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_SET_V_TYPE,
	GEE_HASH_MAP_KEY_SET_V_DUP_FUNC,
	GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC
};
static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_key_set_real_get_element_type (GeeIterable* base);
static GeeIterator* gee_hash_map_key_set_real_iterator (GeeIterable* base);
static gboolean gee_hash_map_key_set_real_add (GeeCollection* base, gconstpointer key);
static void gee_hash_map_key_set_real_clear (GeeCollection* base);
static gboolean gee_hash_map_key_set_real_remove (GeeCollection* base, gconstpointer key);
static gboolean gee_hash_map_key_set_real_contains (GeeCollection* base, gconstpointer key);
static void gee_hash_map_key_set_set_map (GeeHashMapKeySet* self, GeeHashMap* value);
static gpointer gee_hash_map_key_set_parent_class = NULL;
static GeeIterableIface* gee_hash_map_key_set_gee_iterable_parent_iface = NULL;
static GeeCollectionIface* gee_hash_map_key_set_gee_collection_parent_iface = NULL;
static GeeSetIface* gee_hash_map_key_set_gee_set_parent_iface = NULL;
static void gee_hash_map_key_set_finalize (GObject* obj);
static GType gee_hash_map_key_set_get_type (void);
struct _GeeHashMapKeyIteratorPrivate {
	GeeHashMap* _map;
	gint _index;
	GeeHashMapNode* _node;
	gint _stamp;
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

#define GEE_HASH_MAP_KEY_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_KEY_ITERATOR, GeeHashMapKeyIteratorPrivate))
enum  {
	GEE_HASH_MAP_KEY_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_KEY_ITERATOR_MAP,
	GEE_HASH_MAP_KEY_ITERATOR_K_TYPE,
	GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_V_TYPE,
	GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC
};
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static gboolean gee_hash_map_key_iterator_real_next (GeeIterator* base);
static gpointer gee_hash_map_key_iterator_real_get (GeeIterator* base);
static void gee_hash_map_key_iterator_set_map (GeeHashMapKeyIterator* self, GeeHashMap* value);
static gpointer gee_hash_map_key_iterator_parent_class = NULL;
static GeeIteratorIface* gee_hash_map_key_iterator_gee_iterator_parent_iface = NULL;
static void gee_hash_map_key_iterator_finalize (GObject* obj);
static GType gee_hash_map_key_iterator_get_type (void);
struct _GeeHashMapValueCollectionPrivate {
	GeeHashMap* _map;
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

#define GEE_HASH_MAP_VALUE_COLLECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_VALUE_COLLECTION, GeeHashMapValueCollectionPrivate))
enum  {
	GEE_HASH_MAP_VALUE_COLLECTION_DUMMY_PROPERTY,
	GEE_HASH_MAP_VALUE_COLLECTION_MAP,
	GEE_HASH_MAP_VALUE_COLLECTION_SIZE,
	GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE,
	GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE,
	GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC,
	GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC
};
static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GType gee_hash_map_value_collection_real_get_element_type (GeeIterable* base);
static GeeIterator* gee_hash_map_value_collection_real_iterator (GeeIterable* base);
static gboolean gee_hash_map_value_collection_real_add (GeeCollection* base, gconstpointer value);
static void gee_hash_map_value_collection_real_clear (GeeCollection* base);
static gboolean gee_hash_map_value_collection_real_remove (GeeCollection* base, gconstpointer value);
static gboolean gee_hash_map_value_collection_real_contains (GeeCollection* base, gconstpointer value);
static void gee_hash_map_value_collection_set_map (GeeHashMapValueCollection* self, GeeHashMap* value);
static gpointer gee_hash_map_value_collection_parent_class = NULL;
static GeeIterableIface* gee_hash_map_value_collection_gee_iterable_parent_iface = NULL;
static GeeCollectionIface* gee_hash_map_value_collection_gee_collection_parent_iface = NULL;
static void gee_hash_map_value_collection_finalize (GObject* obj);
static GType gee_hash_map_value_collection_get_type (void);
struct _GeeHashMapValueIteratorPrivate {
	GeeHashMap* _map;
	gint _index;
	GeeHashMapNode* _node;
	gint _stamp;
	GType k_type;
	GBoxedCopyFunc k_dup_func;
	GDestroyNotify k_destroy_func;
	GType v_type;
	GBoxedCopyFunc v_dup_func;
	GDestroyNotify v_destroy_func;
};

#define GEE_HASH_MAP_VALUE_ITERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GEE_HASH_MAP_TYPE_VALUE_ITERATOR, GeeHashMapValueIteratorPrivate))
enum  {
	GEE_HASH_MAP_VALUE_ITERATOR_DUMMY_PROPERTY,
	GEE_HASH_MAP_VALUE_ITERATOR_MAP,
	GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE,
	GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE,
	GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC,
	GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC
};
static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map);
static gboolean gee_hash_map_value_iterator_real_next (GeeIterator* base);
static gpointer gee_hash_map_value_iterator_real_get (GeeIterator* base);
static void gee_hash_map_value_iterator_set_map (GeeHashMapValueIterator* self, GeeHashMap* value);
static gpointer gee_hash_map_value_iterator_parent_class = NULL;
static GeeIteratorIface* gee_hash_map_value_iterator_gee_iterator_parent_iface = NULL;
static void gee_hash_map_value_iterator_finalize (GObject* obj);
static GType gee_hash_map_value_iterator_get_type (void);
static gpointer gee_hash_map_parent_class = NULL;
static GeeMapIface* gee_hash_map_gee_map_parent_iface = NULL;
static void gee_hash_map_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



GeeHashMap* gee_hash_map_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GHashFunc key_hash_func, GEqualFunc key_equal_func, GEqualFunc value_equal_func) {
	GParameter * __params;
	GParameter * __params_it;
	GeeHashMap * self;
	__params = g_new0 (GParameter, 6);
	__params_it = __params;
	__params_it->name = "k-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, k_type);
	__params_it++;
	__params_it->name = "k-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_dup_func);
	__params_it++;
	__params_it->name = "k-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_destroy_func);
	__params_it++;
	__params_it->name = "v-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, v_type);
	__params_it++;
	__params_it->name = "v-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_dup_func);
	__params_it++;
	__params_it->name = "v-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	gee_hash_map_set_key_hash_func (self, key_hash_func);
	gee_hash_map_set_key_equal_func (self, key_equal_func);
	gee_hash_map_set_value_equal_func (self, value_equal_func);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


GeeHashMap* gee_hash_map_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GHashFunc key_hash_func, GEqualFunc key_equal_func, GEqualFunc value_equal_func) {
	return gee_hash_map_construct (GEE_TYPE_HASH_MAP, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, key_hash_func, key_equal_func, value_equal_func);
}


static GeeSet* gee_hash_map_real_get_keys (GeeMap* base) {
	GeeHashMap * self;
	self = GEE_HASH_MAP (base);
	return GEE_SET (gee_hash_map_key_set_new (self->priv->k_type, ((GBoxedCopyFunc) (self->priv->k_dup_func)), self->priv->k_destroy_func, self->priv->v_type, ((GBoxedCopyFunc) (self->priv->v_dup_func)), self->priv->v_destroy_func, self));
}


static GeeCollection* gee_hash_map_real_get_values (GeeMap* base) {
	GeeHashMap * self;
	self = GEE_HASH_MAP (base);
	return GEE_COLLECTION (gee_hash_map_value_collection_new (self->priv->k_type, ((GBoxedCopyFunc) (self->priv->k_dup_func)), self->priv->k_destroy_func, self->priv->v_type, ((GBoxedCopyFunc) (self->priv->v_dup_func)), self->priv->v_destroy_func, self));
}


static GeeHashMapNode** gee_hash_map_lookup_node (GeeHashMap* self, gconstpointer key) {
	guint hash_value;
	GeeHashMapNode** node;
	g_return_val_if_fail (GEE_IS_HASH_MAP (self), NULL);
	hash_value = self->priv->_key_hash_func (key);
	node = &self->priv->_nodes[hash_value % self->priv->_array_size];
	while (((*node)) != NULL && (hash_value != ((*node))->key_hash || !self->priv->_key_equal_func (((*node))->key, key))) {
		node = &(((*node))->next);
	}
	return node;
}


static gboolean gee_hash_map_real_contains (GeeMap* base, gconstpointer key) {
	GeeHashMap * self;
	GeeHashMapNode** node;
	self = GEE_HASH_MAP (base);
	node = gee_hash_map_lookup_node (self, key);
	return ((*node) != NULL);
}


static gpointer gee_hash_map_real_get (GeeMap* base, gconstpointer key) {
	GeeHashMap * self;
	GeeHashMapNode* node;
	self = GEE_HASH_MAP (base);
	node = ((*gee_hash_map_lookup_node (self, key)));
	if (node != NULL) {
		gconstpointer _tmp0;
		_tmp0 = NULL;
		return (_tmp0 = node->value, (_tmp0 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->v_dup_func (((gpointer) (_tmp0)))));
	} else {
		return NULL;
	}
}


static void gee_hash_map_real_set (GeeMap* base, gconstpointer key, gconstpointer value) {
	GeeHashMap * self;
	GeeHashMapNode** node;
	self = GEE_HASH_MAP (base);
	node = gee_hash_map_lookup_node (self, key);
	if ((*node) != NULL) {
		gpointer _tmp1;
		gconstpointer _tmp0;
		_tmp1 = NULL;
		_tmp0 = NULL;
		((*node))->value = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->v_dup_func (((gpointer) (_tmp0))))), (((*node))->value == NULL || self->priv->v_destroy_func == NULL ? NULL : (((*node))->value = (self->priv->v_destroy_func (((*node))->value), NULL))), _tmp1);
	} else {
		guint hash_value;
		gconstpointer _tmp3;
		gconstpointer _tmp2;
		hash_value = self->priv->_key_hash_func (key);
		_tmp3 = NULL;
		_tmp2 = NULL;
		(*node) = gee_hash_map_node_new ((_tmp2 = key, (_tmp2 == NULL || self->priv->k_dup_func == NULL ? ((gpointer) (_tmp2)) : self->priv->k_dup_func (((gpointer) (_tmp2))))), (_tmp3 = value, (_tmp3 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) (_tmp3)) : self->priv->v_dup_func (((gpointer) (_tmp3))))), hash_value);
		self->priv->_nnodes++;
		gee_hash_map_resize (self);
	}
	self->priv->_stamp++;
}


static gboolean gee_hash_map_real_remove (GeeMap* base, gconstpointer key) {
	GeeHashMap * self;
	GeeHashMapNode** node;
	self = GEE_HASH_MAP (base);
	node = gee_hash_map_lookup_node (self, key);
	if ((*node) != NULL) {
		gpointer _tmp0;
		gpointer _tmp1;
		_tmp0 = NULL;
		((*node))->key = (_tmp0 = NULL, (((*node))->key == NULL || self->priv->k_destroy_func == NULL ? NULL : (((*node))->key = (self->priv->k_destroy_func (((*node))->key), NULL))), _tmp0);
		_tmp1 = NULL;
		((*node))->value = (_tmp1 = NULL, (((*node))->value == NULL || self->priv->v_destroy_func == NULL ? NULL : (((*node))->value = (self->priv->v_destroy_func (((*node))->value), NULL))), _tmp1);
		(*node) = ((*node))->next;
		self->priv->_nnodes--;
		gee_hash_map_resize (self);
		self->priv->_stamp++;
		return TRUE;
	}
	return FALSE;
}


static void gee_hash_map_real_clear (GeeMap* base) {
	GeeHashMap * self;
	self = GEE_HASH_MAP (base);
	{
		gint i;
		i = 0;
		for (; i < self->priv->_array_size; i++) {
			GeeHashMapNode* _tmp0;
			GeeHashMapNode* node;
			_tmp0 = NULL;
			node = (_tmp0 = self->priv->_nodes[i], self->priv->_nodes[i] = NULL, _tmp0);
			while (node != NULL) {
				GeeHashMapNode* _tmp1;
				GeeHashMapNode* next;
				gpointer _tmp2;
				gpointer _tmp3;
				GeeHashMapNode* _tmp5;
				GeeHashMapNode* _tmp4;
				_tmp1 = NULL;
				next = (_tmp1 = node->next, node->next = NULL, _tmp1);
				_tmp2 = NULL;
				node->key = (_tmp2 = NULL, (node->key == NULL || self->priv->k_destroy_func == NULL ? NULL : (node->key = (self->priv->k_destroy_func (node->key), NULL))), _tmp2);
				_tmp3 = NULL;
				node->value = (_tmp3 = NULL, (node->value == NULL || self->priv->v_destroy_func == NULL ? NULL : (node->value = (self->priv->v_destroy_func (node->value), NULL))), _tmp3);
				_tmp5 = NULL;
				_tmp4 = NULL;
				node = (_tmp5 = (_tmp4 = next, next = NULL, _tmp4), (node == NULL ? NULL : (node = (gee_hash_map_node_free (node), NULL))), _tmp5);
				(next == NULL ? NULL : (next = (gee_hash_map_node_free (next), NULL)));
			}
			(node == NULL ? NULL : (node = (gee_hash_map_node_free (node), NULL)));
		}
	}
	self->priv->_nnodes = 0;
	gee_hash_map_resize (self);
}


static void gee_hash_map_resize (GeeHashMap* self) {
	g_return_if_fail (GEE_IS_HASH_MAP (self));
	if ((self->priv->_array_size >= 3 * self->priv->_nnodes && self->priv->_array_size >= GEE_HASH_MAP_MIN_SIZE) || (3 * self->priv->_array_size <= self->priv->_nnodes && self->priv->_array_size < GEE_HASH_MAP_MAX_SIZE)) {
		gint new_array_size;
		GeeHashMapNode** _tmp0;
		gint new_nodes_length1;
		GeeHashMapNode** new_nodes;
		GeeHashMapNode** _tmp12;
		GeeHashMapNode** _tmp11;
		new_array_size = ((gint) (g_spaced_primes_closest (((guint) (self->priv->_nnodes)))));
		new_array_size = CLAMP (new_array_size, GEE_HASH_MAP_MIN_SIZE, GEE_HASH_MAP_MAX_SIZE);
		_tmp0 = NULL;
		new_nodes = (_tmp0 = g_new0 (GeeHashMapNode*, new_array_size + 1), new_nodes_length1 = new_array_size, _tmp0);
		{
			gint i;
			i = 0;
			for (; i < self->priv->_array_size; i++) {
				GeeHashMapNode* node;
				GeeHashMapNode* next;
				GeeHashMapNode* _tmp4;
				GeeHashMapNode* _tmp3;
				GeeHashMapNode* _tmp2;
				GeeHashMapNode* _tmp1;
				node = NULL;
				next = NULL;
				_tmp4 = NULL;
				_tmp3 = NULL;
				_tmp2 = NULL;
				_tmp1 = NULL;
				for (node = (_tmp2 = (_tmp1 = self->priv->_nodes[i], self->priv->_nodes[i] = NULL, _tmp1), (node == NULL ? NULL : (node = (gee_hash_map_node_free (node), NULL))), _tmp2); node != NULL; node = (_tmp4 = (_tmp3 = next, next = NULL, _tmp3), (node == NULL ? NULL : (node = (gee_hash_map_node_free (node), NULL))), _tmp4)) {
					GeeHashMapNode* _tmp6;
					GeeHashMapNode* _tmp5;
					guint hash_val;
					GeeHashMapNode* _tmp8;
					GeeHashMapNode* _tmp7;
					GeeHashMapNode* _tmp10;
					GeeHashMapNode* _tmp9;
					_tmp6 = NULL;
					_tmp5 = NULL;
					next = (_tmp6 = (_tmp5 = node->next, node->next = NULL, _tmp5), (next == NULL ? NULL : (next = (gee_hash_map_node_free (next), NULL))), _tmp6);
					hash_val = node->key_hash % new_array_size;
					_tmp8 = NULL;
					_tmp7 = NULL;
					node->next = (_tmp8 = (_tmp7 = new_nodes[hash_val], new_nodes[hash_val] = NULL, _tmp7), (node->next == NULL ? NULL : (node->next = (gee_hash_map_node_free (node->next), NULL))), _tmp8);
					_tmp10 = NULL;
					_tmp9 = NULL;
					new_nodes[hash_val] = (_tmp10 = (_tmp9 = node, node = NULL, _tmp9), (new_nodes[hash_val] == NULL ? NULL : (new_nodes[hash_val] = (gee_hash_map_node_free (new_nodes[hash_val]), NULL))), _tmp10);
				}
				(node == NULL ? NULL : (node = (gee_hash_map_node_free (node), NULL)));
				(next == NULL ? NULL : (next = (gee_hash_map_node_free (next), NULL)));
			}
		}
		_tmp12 = NULL;
		_tmp11 = NULL;
		self->priv->_nodes = (_tmp12 = (_tmp11 = new_nodes, new_nodes = NULL, _tmp11), (self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, ((GDestroyNotify) (gee_hash_map_node_free))), NULL)), self->priv->_nodes_length1 = new_nodes_length1, _tmp12);
		self->priv->_array_size = new_array_size;
		new_nodes = (_vala_array_free (new_nodes, new_nodes_length1, ((GDestroyNotify) (gee_hash_map_node_free))), NULL);
	}
}


static gint gee_hash_map_real_get_size (GeeMap* base) {
	GeeHashMap* self;
	self = GEE_HASH_MAP (base);
	return self->priv->_nnodes;
}


void gee_hash_map_set_key_hash_func (GeeHashMap* self, GHashFunc value) {
	g_return_if_fail (GEE_IS_HASH_MAP (self));
	self->priv->_key_hash_func = value;
	g_object_notify (((GObject *) (self)), "key-hash-func");
}


void gee_hash_map_set_key_equal_func (GeeHashMap* self, GEqualFunc value) {
	g_return_if_fail (GEE_IS_HASH_MAP (self));
	self->priv->_key_equal_func = value;
	g_object_notify (((GObject *) (self)), "key-equal-func");
}


void gee_hash_map_set_value_equal_func (GeeHashMap* self, GEqualFunc value) {
	g_return_if_fail (GEE_IS_HASH_MAP (self));
	self->priv->_value_equal_func = value;
	g_object_notify (((GObject *) (self)), "value-equal-func");
}


static GObject * gee_hash_map_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	GeeHashMapClass * klass;
	GObjectClass * parent_class;
	GeeHashMap * self;
	klass = GEE_HASH_MAP_CLASS (g_type_class_peek (GEE_TYPE_HASH_MAP));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = GEE_HASH_MAP (obj);
	{
		GeeHashMapNode** _tmp0;
		self->priv->_array_size = GEE_HASH_MAP_MIN_SIZE;
		_tmp0 = NULL;
		self->priv->_nodes = (_tmp0 = g_new0 (GeeHashMapNode*, self->priv->_array_size + 1), (self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, ((GDestroyNotify) (gee_hash_map_node_free))), NULL)), self->priv->_nodes_length1 = self->priv->_array_size, _tmp0);
	}
	return obj;
}


static GeeHashMapNode* gee_hash_map_node_new (gpointer k, gpointer v, guint hash) {
	GeeHashMapNode* self;
	gpointer _tmp0;
	gpointer _tmp1;
	self = g_slice_new0 (GeeHashMapNode);
	gee_hash_map_node_instance_init (self);
	_tmp0 = NULL;
	self->key = (_tmp0 = k, k = NULL, _tmp0);
	_tmp1 = NULL;
	self->value = (_tmp1 = v, v = NULL, _tmp1);
	self->key_hash = hash;
	return self;
}


static void gee_hash_map_node_instance_init (GeeHashMapNode * self) {
}


static void gee_hash_map_node_free (GeeHashMapNode* self) {
	(self->next == NULL ? NULL : (self->next = (gee_hash_map_node_free (self->next), NULL)));
	g_slice_free (GeeHashMapNode, self);
}


static GeeHashMapKeySet* gee_hash_map_key_set_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GParameter * __params;
	GParameter * __params_it;
	GeeHashMapKeySet * self;
	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
	__params = g_new0 (GParameter, 6);
	__params_it = __params;
	__params_it->name = "k-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, k_type);
	__params_it++;
	__params_it->name = "k-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_dup_func);
	__params_it++;
	__params_it->name = "k-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_destroy_func);
	__params_it++;
	__params_it->name = "v-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, v_type);
	__params_it++;
	__params_it->name = "v-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_dup_func);
	__params_it++;
	__params_it->name = "v-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	gee_hash_map_key_set_set_map (self, map);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static GeeHashMapKeySet* gee_hash_map_key_set_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_key_set_construct (GEE_HASH_MAP_TYPE_KEY_SET, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GType gee_hash_map_key_set_real_get_element_type (GeeIterable* base) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (base);
	return self->priv->k_type;
}


static GeeIterator* gee_hash_map_key_set_real_iterator (GeeIterable* base) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (base);
	return GEE_ITERATOR (gee_hash_map_key_iterator_new (self->priv->k_type, ((GBoxedCopyFunc) (self->priv->k_dup_func)), self->priv->k_destroy_func, self->priv->v_type, ((GBoxedCopyFunc) (self->priv->v_dup_func)), self->priv->v_destroy_func, self->priv->_map));
}


static gboolean gee_hash_map_key_set_real_add (GeeCollection* base, gconstpointer key) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (base);
	g_assert_not_reached ();
}


static void gee_hash_map_key_set_real_clear (GeeCollection* base) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (base);
	g_assert_not_reached ();
}


static gboolean gee_hash_map_key_set_real_remove (GeeCollection* base, gconstpointer key) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (base);
	g_assert_not_reached ();
}


static gboolean gee_hash_map_key_set_real_contains (GeeCollection* base, gconstpointer key) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (base);
	return gee_map_contains (GEE_MAP (self->priv->_map), key);
}


static void gee_hash_map_key_set_set_map (GeeHashMapKeySet* self, GeeHashMap* value) {
	GeeHashMap* _tmp1;
	GeeHashMap* _tmp0;
	g_return_if_fail (GEE_HASH_MAP_IS_KEY_SET (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_map = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL))), _tmp1);
	g_object_notify (((GObject *) (self)), "map");
}


static gint gee_hash_map_key_set_real_get_size (GeeCollection* base) {
	GeeHashMapKeySet* self;
	self = GEE_HASH_MAP_KEY_SET (base);
	return gee_map_get_size (GEE_MAP (self->priv->_map));
}


static void gee_hash_map_key_set_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (object);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_SET_SIZE:
		g_value_set_int (value, gee_collection_get_size (GEE_COLLECTION (self)));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_key_set_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (object);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_SET_MAP:
		gee_hash_map_key_set_set_map (self, g_value_get_object (value));
		break;
		case GEE_HASH_MAP_KEY_SET_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_key_set_class_init (GeeHashMapKeySetClass * klass) {
	gee_hash_map_key_set_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapKeySetPrivate));
	G_OBJECT_CLASS (klass)->get_property = gee_hash_map_key_set_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_hash_map_key_set_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_key_set_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_MAP, g_param_spec_object ("map", "map", "map", GEE_TYPE_HASH_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_SET_SIZE, "size");
}


static void gee_hash_map_key_set_gee_iterable_interface_init (GeeIterableIface * iface) {
	gee_hash_map_key_set_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_element_type = gee_hash_map_key_set_real_get_element_type;
	iface->iterator = gee_hash_map_key_set_real_iterator;
}


static void gee_hash_map_key_set_gee_collection_interface_init (GeeCollectionIface * iface) {
	gee_hash_map_key_set_gee_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->add = gee_hash_map_key_set_real_add;
	iface->clear = gee_hash_map_key_set_real_clear;
	iface->remove = gee_hash_map_key_set_real_remove;
	iface->contains = gee_hash_map_key_set_real_contains;
	iface->get_size = gee_hash_map_key_set_real_get_size;
}


static void gee_hash_map_key_set_gee_set_interface_init (GeeSetIface * iface) {
	gee_hash_map_key_set_gee_set_parent_iface = g_type_interface_peek_parent (iface);
}


static void gee_hash_map_key_set_instance_init (GeeHashMapKeySet * self) {
	self->priv = GEE_HASH_MAP_KEY_SET_GET_PRIVATE (self);
}


static void gee_hash_map_key_set_finalize (GObject* obj) {
	GeeHashMapKeySet * self;
	self = GEE_HASH_MAP_KEY_SET (obj);
	(self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL)));
	G_OBJECT_CLASS (gee_hash_map_key_set_parent_class)->finalize (obj);
}


static GType gee_hash_map_key_set_get_type (void) {
	static GType gee_hash_map_key_set_type_id = 0;
	if (gee_hash_map_key_set_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapKeySetClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_key_set_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapKeySet), 0, (GInstanceInitFunc) gee_hash_map_key_set_instance_init, NULL };
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) gee_hash_map_key_set_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_collection_info = { (GInterfaceInitFunc) gee_hash_map_key_set_gee_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_set_info = { (GInterfaceInitFunc) gee_hash_map_key_set_gee_set_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_hash_map_key_set_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMapKeySet", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_key_set_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
		g_type_add_interface_static (gee_hash_map_key_set_type_id, GEE_TYPE_COLLECTION, &gee_collection_info);
		g_type_add_interface_static (gee_hash_map_key_set_type_id, GEE_TYPE_SET, &gee_set_info);
	}
	return gee_hash_map_key_set_type_id;
}


static GeeHashMapKeyIterator* gee_hash_map_key_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GParameter * __params;
	GParameter * __params_it;
	GeeHashMapKeyIterator * self;
	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
	__params = g_new0 (GParameter, 6);
	__params_it = __params;
	__params_it->name = "k-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, k_type);
	__params_it++;
	__params_it->name = "k-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_dup_func);
	__params_it++;
	__params_it->name = "k-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_destroy_func);
	__params_it++;
	__params_it->name = "v-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, v_type);
	__params_it++;
	__params_it->name = "v-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_dup_func);
	__params_it++;
	__params_it->name = "v-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	gee_hash_map_key_iterator_set_map (self, map);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static GeeHashMapKeyIterator* gee_hash_map_key_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_key_iterator_construct (GEE_HASH_MAP_TYPE_KEY_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static gboolean gee_hash_map_key_iterator_real_next (GeeIterator* base) {
	GeeHashMapKeyIterator * self;
	self = GEE_HASH_MAP_KEY_ITERATOR (base);
	if (self->priv->_node != NULL) {
		self->priv->_node = self->priv->_node->next;
	}
	while (self->priv->_node == NULL && self->priv->_index + 1 < self->priv->_map->priv->_array_size) {
		self->priv->_index++;
		self->priv->_node = self->priv->_map->priv->_nodes[self->priv->_index];
	}
	return (self->priv->_node != NULL);
}


static gpointer gee_hash_map_key_iterator_real_get (GeeIterator* base) {
	GeeHashMapKeyIterator * self;
	gconstpointer _tmp0;
	self = GEE_HASH_MAP_KEY_ITERATOR (base);
	g_assert (self->priv->_stamp == self->priv->_map->priv->_stamp);
	g_assert (self->priv->_node != NULL);
	_tmp0 = NULL;
	return (_tmp0 = self->priv->_node->key, (_tmp0 == NULL || self->priv->k_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->k_dup_func (((gpointer) (_tmp0)))));
}


static void gee_hash_map_key_iterator_set_map (GeeHashMapKeyIterator* self, GeeHashMap* value) {
	GeeHashMap* _tmp1;
	GeeHashMap* _tmp0;
	g_return_if_fail (GEE_HASH_MAP_IS_KEY_ITERATOR (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_map = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL))), _tmp1);
	self->priv->_stamp = self->priv->_map->priv->_stamp;
	g_object_notify (((GObject *) (self)), "map");
}


static void gee_hash_map_key_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapKeyIterator * self;
	self = GEE_HASH_MAP_KEY_ITERATOR (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_key_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapKeyIterator * self;
	self = GEE_HASH_MAP_KEY_ITERATOR (object);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_ITERATOR_MAP:
		gee_hash_map_key_iterator_set_map (self, g_value_get_object (value));
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_key_iterator_class_init (GeeHashMapKeyIteratorClass * klass) {
	gee_hash_map_key_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapKeyIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = gee_hash_map_key_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_hash_map_key_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_key_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_ITERATOR_MAP, g_param_spec_object ("map", "map", "map", GEE_TYPE_HASH_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
}


static void gee_hash_map_key_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_hash_map_key_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = gee_hash_map_key_iterator_real_next;
	iface->get = gee_hash_map_key_iterator_real_get;
}


static void gee_hash_map_key_iterator_instance_init (GeeHashMapKeyIterator * self) {
	self->priv = GEE_HASH_MAP_KEY_ITERATOR_GET_PRIVATE (self);
	self->priv->_index = -1;
}


static void gee_hash_map_key_iterator_finalize (GObject* obj) {
	GeeHashMapKeyIterator * self;
	self = GEE_HASH_MAP_KEY_ITERATOR (obj);
	(self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL)));
	G_OBJECT_CLASS (gee_hash_map_key_iterator_parent_class)->finalize (obj);
}


static GType gee_hash_map_key_iterator_get_type (void) {
	static GType gee_hash_map_key_iterator_type_id = 0;
	if (gee_hash_map_key_iterator_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapKeyIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_key_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapKeyIterator), 0, (GInstanceInitFunc) gee_hash_map_key_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_key_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_hash_map_key_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMapKeyIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_key_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	}
	return gee_hash_map_key_iterator_type_id;
}


static GeeHashMapValueCollection* gee_hash_map_value_collection_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GParameter * __params;
	GParameter * __params_it;
	GeeHashMapValueCollection * self;
	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
	__params = g_new0 (GParameter, 6);
	__params_it = __params;
	__params_it->name = "k-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, k_type);
	__params_it++;
	__params_it->name = "k-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_dup_func);
	__params_it++;
	__params_it->name = "k-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_destroy_func);
	__params_it++;
	__params_it->name = "v-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, v_type);
	__params_it++;
	__params_it->name = "v-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_dup_func);
	__params_it++;
	__params_it->name = "v-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	gee_hash_map_value_collection_set_map (self, map);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static GeeHashMapValueCollection* gee_hash_map_value_collection_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_value_collection_construct (GEE_HASH_MAP_TYPE_VALUE_COLLECTION, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static GType gee_hash_map_value_collection_real_get_element_type (GeeIterable* base) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	return self->priv->v_type;
}


static GeeIterator* gee_hash_map_value_collection_real_iterator (GeeIterable* base) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	return GEE_ITERATOR (gee_hash_map_value_iterator_new (self->priv->k_type, ((GBoxedCopyFunc) (self->priv->k_dup_func)), self->priv->k_destroy_func, self->priv->v_type, ((GBoxedCopyFunc) (self->priv->v_dup_func)), self->priv->v_destroy_func, self->priv->_map));
}


static gboolean gee_hash_map_value_collection_real_add (GeeCollection* base, gconstpointer value) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	g_assert_not_reached ();
}


static void gee_hash_map_value_collection_real_clear (GeeCollection* base) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	g_assert_not_reached ();
}


static gboolean gee_hash_map_value_collection_real_remove (GeeCollection* base, gconstpointer value) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	g_assert_not_reached ();
}


static gboolean gee_hash_map_value_collection_real_contains (GeeCollection* base, gconstpointer value) {
	GeeHashMapValueCollection * self;
	GeeIterator* it;
	gboolean _tmp3;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	it = gee_iterable_iterator (GEE_ITERABLE (self));
	while (gee_iterator_next (it)) {
		gpointer _tmp0;
		gboolean _tmp1;
		_tmp0 = NULL;
		if ((_tmp1 = self->priv->_map->priv->_value_equal_func ((_tmp0 = gee_iterator_get (it)), value), (_tmp0 == NULL || self->priv->v_destroy_func == NULL ? NULL : (_tmp0 = (self->priv->v_destroy_func (_tmp0), NULL))), _tmp1)) {
			gboolean _tmp2;
			return (_tmp2 = TRUE, (it == NULL ? NULL : (it = (g_object_unref (it), NULL))), _tmp2);
		}
	}
	return (_tmp3 = FALSE, (it == NULL ? NULL : (it = (g_object_unref (it), NULL))), _tmp3);
}


static void gee_hash_map_value_collection_set_map (GeeHashMapValueCollection* self, GeeHashMap* value) {
	GeeHashMap* _tmp1;
	GeeHashMap* _tmp0;
	g_return_if_fail (GEE_HASH_MAP_IS_VALUE_COLLECTION (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_map = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL))), _tmp1);
	g_object_notify (((GObject *) (self)), "map");
}


static gint gee_hash_map_value_collection_real_get_size (GeeCollection* base) {
	GeeHashMapValueCollection* self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (base);
	return gee_map_get_size (GEE_MAP (self->priv->_map));
}


static void gee_hash_map_value_collection_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (object);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_COLLECTION_SIZE:
		g_value_set_int (value, gee_collection_get_size (GEE_COLLECTION (self)));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_value_collection_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (object);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_COLLECTION_MAP:
		gee_hash_map_value_collection_set_map (self, g_value_get_object (value));
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_value_collection_class_init (GeeHashMapValueCollectionClass * klass) {
	gee_hash_map_value_collection_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapValueCollectionPrivate));
	G_OBJECT_CLASS (klass)->get_property = gee_hash_map_value_collection_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_hash_map_value_collection_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_value_collection_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_MAP, g_param_spec_object ("map", "map", "map", GEE_TYPE_HASH_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_COLLECTION_SIZE, "size");
}


static void gee_hash_map_value_collection_gee_iterable_interface_init (GeeIterableIface * iface) {
	gee_hash_map_value_collection_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_element_type = gee_hash_map_value_collection_real_get_element_type;
	iface->iterator = gee_hash_map_value_collection_real_iterator;
}


static void gee_hash_map_value_collection_gee_collection_interface_init (GeeCollectionIface * iface) {
	gee_hash_map_value_collection_gee_collection_parent_iface = g_type_interface_peek_parent (iface);
	iface->add = gee_hash_map_value_collection_real_add;
	iface->clear = gee_hash_map_value_collection_real_clear;
	iface->remove = gee_hash_map_value_collection_real_remove;
	iface->contains = gee_hash_map_value_collection_real_contains;
	iface->get_size = gee_hash_map_value_collection_real_get_size;
}


static void gee_hash_map_value_collection_instance_init (GeeHashMapValueCollection * self) {
	self->priv = GEE_HASH_MAP_VALUE_COLLECTION_GET_PRIVATE (self);
}


static void gee_hash_map_value_collection_finalize (GObject* obj) {
	GeeHashMapValueCollection * self;
	self = GEE_HASH_MAP_VALUE_COLLECTION (obj);
	(self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL)));
	G_OBJECT_CLASS (gee_hash_map_value_collection_parent_class)->finalize (obj);
}


static GType gee_hash_map_value_collection_get_type (void) {
	static GType gee_hash_map_value_collection_type_id = 0;
	if (gee_hash_map_value_collection_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapValueCollectionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_value_collection_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapValueCollection), 0, (GInstanceInitFunc) gee_hash_map_value_collection_instance_init, NULL };
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) gee_hash_map_value_collection_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo gee_collection_info = { (GInterfaceInitFunc) gee_hash_map_value_collection_gee_collection_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_hash_map_value_collection_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMapValueCollection", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_value_collection_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
		g_type_add_interface_static (gee_hash_map_value_collection_type_id, GEE_TYPE_COLLECTION, &gee_collection_info);
	}
	return gee_hash_map_value_collection_type_id;
}


static GeeHashMapValueIterator* gee_hash_map_value_iterator_construct (GType object_type, GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	GParameter * __params;
	GParameter * __params_it;
	GeeHashMapValueIterator * self;
	g_return_val_if_fail (GEE_IS_HASH_MAP (map), NULL);
	__params = g_new0 (GParameter, 6);
	__params_it = __params;
	__params_it->name = "k-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, k_type);
	__params_it++;
	__params_it->name = "k-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_dup_func);
	__params_it++;
	__params_it->name = "k-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, k_destroy_func);
	__params_it++;
	__params_it->name = "v-type";
	g_value_init (&__params_it->value, G_TYPE_GTYPE);
	g_value_set_gtype (&__params_it->value, v_type);
	__params_it++;
	__params_it->name = "v-dup-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_dup_func);
	__params_it++;
	__params_it->name = "v-destroy-func";
	g_value_init (&__params_it->value, G_TYPE_POINTER);
	g_value_set_pointer (&__params_it->value, v_destroy_func);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	gee_hash_map_value_iterator_set_map (self, map);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


static GeeHashMapValueIterator* gee_hash_map_value_iterator_new (GType k_type, GBoxedCopyFunc k_dup_func, GDestroyNotify k_destroy_func, GType v_type, GBoxedCopyFunc v_dup_func, GDestroyNotify v_destroy_func, GeeHashMap* map) {
	return gee_hash_map_value_iterator_construct (GEE_HASH_MAP_TYPE_VALUE_ITERATOR, k_type, k_dup_func, k_destroy_func, v_type, v_dup_func, v_destroy_func, map);
}


static gboolean gee_hash_map_value_iterator_real_next (GeeIterator* base) {
	GeeHashMapValueIterator * self;
	self = GEE_HASH_MAP_VALUE_ITERATOR (base);
	if (self->priv->_node != NULL) {
		self->priv->_node = self->priv->_node->next;
	}
	while (self->priv->_node == NULL && self->priv->_index + 1 < self->priv->_map->priv->_array_size) {
		self->priv->_index++;
		self->priv->_node = self->priv->_map->priv->_nodes[self->priv->_index];
	}
	return (self->priv->_node != NULL);
}


static gpointer gee_hash_map_value_iterator_real_get (GeeIterator* base) {
	GeeHashMapValueIterator * self;
	gconstpointer _tmp0;
	self = GEE_HASH_MAP_VALUE_ITERATOR (base);
	g_assert (self->priv->_stamp == self->priv->_map->priv->_stamp);
	g_assert (self->priv->_node != NULL);
	_tmp0 = NULL;
	return (_tmp0 = self->priv->_node->value, (_tmp0 == NULL || self->priv->v_dup_func == NULL ? ((gpointer) (_tmp0)) : self->priv->v_dup_func (((gpointer) (_tmp0)))));
}


static void gee_hash_map_value_iterator_set_map (GeeHashMapValueIterator* self, GeeHashMap* value) {
	GeeHashMap* _tmp1;
	GeeHashMap* _tmp0;
	g_return_if_fail (GEE_HASH_MAP_IS_VALUE_ITERATOR (self));
	_tmp1 = NULL;
	_tmp0 = NULL;
	self->priv->_map = (_tmp1 = (_tmp0 = value, (_tmp0 == NULL ? NULL : g_object_ref (_tmp0))), (self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL))), _tmp1);
	self->priv->_stamp = self->priv->_map->priv->_stamp;
	g_object_notify (((GObject *) (self)), "map");
}


static void gee_hash_map_value_iterator_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMapValueIterator * self;
	self = GEE_HASH_MAP_VALUE_ITERATOR (object);
	switch (property_id) {
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_value_iterator_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMapValueIterator * self;
	self = GEE_HASH_MAP_VALUE_ITERATOR (object);
	switch (property_id) {
		case GEE_HASH_MAP_VALUE_ITERATOR_MAP:
		gee_hash_map_value_iterator_set_map (self, g_value_get_object (value));
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_value_iterator_class_init (GeeHashMapValueIteratorClass * klass) {
	gee_hash_map_value_iterator_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapValueIteratorPrivate));
	G_OBJECT_CLASS (klass)->get_property = gee_hash_map_value_iterator_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_hash_map_value_iterator_set_property;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_value_iterator_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_ITERATOR_MAP, g_param_spec_object ("map", "map", "map", GEE_TYPE_HASH_MAP, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
}


static void gee_hash_map_value_iterator_gee_iterator_interface_init (GeeIteratorIface * iface) {
	gee_hash_map_value_iterator_gee_iterator_parent_iface = g_type_interface_peek_parent (iface);
	iface->next = gee_hash_map_value_iterator_real_next;
	iface->get = gee_hash_map_value_iterator_real_get;
}


static void gee_hash_map_value_iterator_instance_init (GeeHashMapValueIterator * self) {
	self->priv = GEE_HASH_MAP_VALUE_ITERATOR_GET_PRIVATE (self);
	self->priv->_index = -1;
}


static void gee_hash_map_value_iterator_finalize (GObject* obj) {
	GeeHashMapValueIterator * self;
	self = GEE_HASH_MAP_VALUE_ITERATOR (obj);
	(self->priv->_map == NULL ? NULL : (self->priv->_map = (g_object_unref (self->priv->_map), NULL)));
	G_OBJECT_CLASS (gee_hash_map_value_iterator_parent_class)->finalize (obj);
}


static GType gee_hash_map_value_iterator_get_type (void) {
	static GType gee_hash_map_value_iterator_type_id = 0;
	if (gee_hash_map_value_iterator_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapValueIteratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_value_iterator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMapValueIterator), 0, (GInstanceInitFunc) gee_hash_map_value_iterator_instance_init, NULL };
		static const GInterfaceInfo gee_iterator_info = { (GInterfaceInitFunc) gee_hash_map_value_iterator_gee_iterator_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_hash_map_value_iterator_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMapValueIterator", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_value_iterator_type_id, GEE_TYPE_ITERATOR, &gee_iterator_info);
	}
	return gee_hash_map_value_iterator_type_id;
}


static void gee_hash_map_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	GeeHashMap * self;
	self = GEE_HASH_MAP (object);
	switch (property_id) {
		case GEE_HASH_MAP_SIZE:
		g_value_set_int (value, gee_map_get_size (GEE_MAP (self)));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	GeeHashMap * self;
	self = GEE_HASH_MAP (object);
	switch (property_id) {
		case GEE_HASH_MAP_KEY_HASH_FUNC:
		gee_hash_map_set_key_hash_func (self, g_value_get_pointer (value));
		break;
		case GEE_HASH_MAP_KEY_EQUAL_FUNC:
		gee_hash_map_set_key_equal_func (self, g_value_get_pointer (value));
		break;
		case GEE_HASH_MAP_VALUE_EQUAL_FUNC:
		gee_hash_map_set_value_equal_func (self, g_value_get_pointer (value));
		break;
		case GEE_HASH_MAP_K_TYPE:
		self->priv->k_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_K_DUP_FUNC:
		self->priv->k_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_K_DESTROY_FUNC:
		self->priv->k_destroy_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_V_TYPE:
		self->priv->v_type = g_value_get_gtype (value);
		break;
		case GEE_HASH_MAP_V_DUP_FUNC:
		self->priv->v_dup_func = g_value_get_pointer (value);
		break;
		case GEE_HASH_MAP_V_DESTROY_FUNC:
		self->priv->v_destroy_func = g_value_get_pointer (value);
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void gee_hash_map_class_init (GeeHashMapClass * klass) {
	gee_hash_map_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (GeeHashMapPrivate));
	G_OBJECT_CLASS (klass)->get_property = gee_hash_map_get_property;
	G_OBJECT_CLASS (klass)->set_property = gee_hash_map_set_property;
	G_OBJECT_CLASS (klass)->constructor = gee_hash_map_constructor;
	G_OBJECT_CLASS (klass)->finalize = gee_hash_map_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_TYPE, g_param_spec_gtype ("k-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_DUP_FUNC, g_param_spec_pointer ("k-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_K_DESTROY_FUNC, g_param_spec_pointer ("k-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_TYPE, g_param_spec_gtype ("v-type", "type", "type", G_TYPE_NONE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_DUP_FUNC, g_param_spec_pointer ("v-dup-func", "dup func", "dup func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_V_DESTROY_FUNC, g_param_spec_pointer ("v-destroy-func", "destroy func", "destroy func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_override_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_SIZE, "size");
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_HASH_FUNC, g_param_spec_pointer ("key-hash-func", "key-hash-func", "key-hash-func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_KEY_EQUAL_FUNC, g_param_spec_pointer ("key-equal-func", "key-equal-func", "key-equal-func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), GEE_HASH_MAP_VALUE_EQUAL_FUNC, g_param_spec_pointer ("value-equal-func", "value-equal-func", "value-equal-func", G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE));
}


static void gee_hash_map_gee_map_interface_init (GeeMapIface * iface) {
	gee_hash_map_gee_map_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_keys = gee_hash_map_real_get_keys;
	iface->get_values = gee_hash_map_real_get_values;
	iface->contains = gee_hash_map_real_contains;
	iface->get = gee_hash_map_real_get;
	iface->set = gee_hash_map_real_set;
	iface->remove = gee_hash_map_real_remove;
	iface->clear = gee_hash_map_real_clear;
	iface->get_size = gee_hash_map_real_get_size;
}


static void gee_hash_map_instance_init (GeeHashMap * self) {
	self->priv = GEE_HASH_MAP_GET_PRIVATE (self);
	self->priv->_stamp = 0;
}


static void gee_hash_map_finalize (GObject* obj) {
	GeeHashMap * self;
	self = GEE_HASH_MAP (obj);
	{
		gee_map_clear (GEE_MAP (self));
	}
	self->priv->_nodes = (_vala_array_free (self->priv->_nodes, self->priv->_nodes_length1, ((GDestroyNotify) (gee_hash_map_node_free))), NULL);
	G_OBJECT_CLASS (gee_hash_map_parent_class)->finalize (obj);
}


GType gee_hash_map_get_type (void) {
	static GType gee_hash_map_type_id = 0;
	if (gee_hash_map_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeHashMapClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) gee_hash_map_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (GeeHashMap), 0, (GInstanceInitFunc) gee_hash_map_instance_init, NULL };
		static const GInterfaceInfo gee_map_info = { (GInterfaceInitFunc) gee_hash_map_gee_map_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		gee_hash_map_type_id = g_type_register_static (G_TYPE_OBJECT, "GeeHashMap", &g_define_type_info, 0);
		g_type_add_interface_static (gee_hash_map_type_id, GEE_TYPE_MAP, &gee_map_info);
	}
	return gee_hash_map_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if (array != NULL && destroy_func != NULL) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) (array))[i] != NULL)
			destroy_func (((gpointer*) (array))[i]);
		}
		else
		for (i = 0; ((gpointer*) (array))[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) (array))[i]);
		}
	}
	g_free (array);
}




