/* testhashset.vala
 *
 * Copyright (C) 2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <tests/testhashset.h>
#include <gee/hashset.h>
#include <stdlib.h>
#include <string.h>
#include <gee/collection.h>
#include <gee/iterable.h>
#include <gee/iterator.h>




static void test_hashset_add (void);
static void test_hashset_clear (void);
static void test_hashset_contains (void);
static void test_hashset_remove (void);
static void test_hashset_size (void);
static void test_hashset_iterator (void);
static void _test_hashset_add_gcallback (void);
static void _test_hashset_clear_gcallback (void);
static void _test_hashset_contains_gcallback (void);
static void _test_hashset_remove_gcallback (void);
static void _test_hashset_size_gcallback (void);
static void _test_hashset_iterator_gcallback (void);
static void _main (char** args, int args_length1);



static void test_hashset_add (void) {
	GeeHashSet* hashset;
	GeeHashSet* hashsetInt;
	GeeHashSet* hashsetObject;
	GObject* fooObject;
	GObject* fooObject2;
	/* Check adding of strings*/
	hashset = gee_hash_set_new (G_TYPE_STRING, ((GBoxedCopyFunc) (g_strdup)), g_free, g_str_hash, g_str_equal);
	gee_collection_add (GEE_COLLECTION (hashset), "42");
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), "42"));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 1);
	gee_collection_add (GEE_COLLECTION (hashset), "43");
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), "42"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), "43"));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 2);
	/* Check add same element*/
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 2);
	gee_collection_add (GEE_COLLECTION (hashset), "43");
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), "42"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), "43"));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 2);
	/* Check adding of ints*/
	hashsetInt = gee_hash_set_new (G_TYPE_INT, NULL, NULL, g_direct_hash, g_direct_equal);
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (42));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (42)));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 1);
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (42)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43)));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 2);
	/* Check add same element*/
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 2);
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (42)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43)));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 2);
	/* Check adding of objects*/
	hashsetObject = gee_hash_set_new (G_TYPE_OBJECT, ((GBoxedCopyFunc) (g_object_ref)), g_object_unref, g_direct_hash, g_direct_equal);
	fooObject = g_object_new (G_TYPE_OBJECT, NULL);
	gee_collection_add (GEE_COLLECTION (hashsetObject), fooObject);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetObject), fooObject));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetObject)) == 1);
	fooObject2 = g_object_new (G_TYPE_OBJECT, NULL);
	gee_collection_add (GEE_COLLECTION (hashsetObject), fooObject2);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetObject), fooObject));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetObject), fooObject2));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetObject)) == 2);
	/* Check add same element*/
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetObject)) == 2);
	gee_collection_add (GEE_COLLECTION (hashsetObject), fooObject2);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetObject), fooObject));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetObject), fooObject2));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetObject)) == 2);
	(hashset == NULL ? NULL : (hashset = (g_object_unref (hashset), NULL)));
	(hashsetInt == NULL ? NULL : (hashsetInt = (g_object_unref (hashsetInt), NULL)));
	(hashsetObject == NULL ? NULL : (hashsetObject = (g_object_unref (hashsetObject), NULL)));
	(fooObject == NULL ? NULL : (fooObject = (g_object_unref (fooObject), NULL)));
	(fooObject2 == NULL ? NULL : (fooObject2 = (g_object_unref (fooObject2), NULL)));
}


static void test_hashset_clear (void) {
	GeeHashSet* hashset;
	hashset = gee_hash_set_new (G_TYPE_STRING, ((GBoxedCopyFunc) (g_strdup)), g_free, g_str_hash, g_str_equal);
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 0);
	/* Check clear on empty set*/
	gee_collection_clear (GEE_COLLECTION (hashset));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 0);
	/* Check clear one item*/
	gee_collection_add (GEE_COLLECTION (hashset), "1");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 1);
	gee_collection_clear (GEE_COLLECTION (hashset));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 0);
	/* Check clear multiple items*/
	gee_collection_add (GEE_COLLECTION (hashset), "1");
	gee_collection_add (GEE_COLLECTION (hashset), "2");
	gee_collection_add (GEE_COLLECTION (hashset), "3");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 3);
	gee_collection_clear (GEE_COLLECTION (hashset));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 0);
	(hashset == NULL ? NULL : (hashset = (g_object_unref (hashset), NULL)));
}


static void test_hashset_contains (void) {
	GeeHashSet* hashsetString;
	GeeHashSet* hashsetInt;
	hashsetString = gee_hash_set_new (G_TYPE_STRING, ((GBoxedCopyFunc) (g_strdup)), g_free, g_str_hash, g_str_equal);
	/* Check on empty set*/
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "1"));
	/* Check items*/
	gee_collection_add (GEE_COLLECTION (hashsetString), "10");
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "10"));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "20"));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "30"));
	gee_collection_add (GEE_COLLECTION (hashsetString), "20");
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "10"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "20"));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "30"));
	gee_collection_add (GEE_COLLECTION (hashsetString), "30");
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "10"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "20"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "30"));
	/* Clear and recheck*/
	gee_collection_clear (GEE_COLLECTION (hashsetString));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "10"));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "20"));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetString), "30"));
	hashsetInt = gee_hash_set_new (G_TYPE_INT, NULL, NULL, g_direct_hash, g_direct_equal);
	/* Check items*/
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (10));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (10)));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (20)));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (30)));
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (20));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (10)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (20)));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (30)));
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (30));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (10)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (20)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (30)));
	/* Clear and recheck*/
	gee_collection_clear (GEE_COLLECTION (hashsetInt));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (10)));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (20)));
	g_assert (!gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (30)));
	(hashsetString == NULL ? NULL : (hashsetString = (g_object_unref (hashsetString), NULL)));
	(hashsetInt == NULL ? NULL : (hashsetInt = (g_object_unref (hashsetInt), NULL)));
}


static void test_hashset_remove (void) {
	GeeHashSet* hashsetString;
	GeeHashSet* hashsetInt;
	hashsetString = gee_hash_set_new (G_TYPE_STRING, ((GBoxedCopyFunc) (g_strdup)), g_free, g_str_hash, g_str_equal);
	/* Check remove if list is empty*/
	gee_collection_remove (GEE_COLLECTION (hashsetString), "42");
	/* Add 5 different elements*/
	gee_collection_add (GEE_COLLECTION (hashsetString), "42");
	gee_collection_add (GEE_COLLECTION (hashsetString), "43");
	gee_collection_add (GEE_COLLECTION (hashsetString), "44");
	gee_collection_add (GEE_COLLECTION (hashsetString), "45");
	gee_collection_add (GEE_COLLECTION (hashsetString), "46");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetString)) == 5);
	/* Check remove first*/
	gee_collection_remove (GEE_COLLECTION (hashsetString), "42");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetString)) == 4);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "43"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "44"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "45"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "46"));
	/* Check remove last*/
	gee_collection_remove (GEE_COLLECTION (hashsetString), "46");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetString)) == 3);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "43"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "44"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "45"));
	/* Check remove in between*/
	gee_collection_remove (GEE_COLLECTION (hashsetString), "44");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetString)) == 2);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "43"));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetString), "45"));
	/* Check removing of int element*/
	hashsetInt = gee_hash_set_new (G_TYPE_INT, NULL, NULL, g_direct_hash, g_direct_equal);
	/* Add 5 different elements*/
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (42));
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43));
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (44));
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (45));
	gee_collection_add (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (46));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 5);
	/* Remove first*/
	gee_collection_remove (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (42));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 4);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (44)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (45)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (46)));
	/* Remove last*/
	gee_collection_remove (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (46));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 3);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (44)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (45)));
	/* Remove in between*/
	gee_collection_remove (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (44));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashsetInt)) == 2);
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (43)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashsetInt), GINT_TO_POINTER (45)));
	(hashsetString == NULL ? NULL : (hashsetString = (g_object_unref (hashsetString), NULL)));
	(hashsetInt == NULL ? NULL : (hashsetInt = (g_object_unref (hashsetInt), NULL)));
}


static void test_hashset_size (void) {
	GeeHashSet* hashset;
	hashset = gee_hash_set_new (G_TYPE_STRING, ((GBoxedCopyFunc) (g_strdup)), g_free, g_str_hash, g_str_equal);
	/* Check empty list*/
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 0);
	/* Check when one item*/
	gee_collection_add (GEE_COLLECTION (hashset), "1");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 1);
	/* Check when more items*/
	gee_collection_add (GEE_COLLECTION (hashset), "2");
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 2);
	/* Check when items cleared*/
	gee_collection_clear (GEE_COLLECTION (hashset));
	g_assert (gee_collection_get_size (GEE_COLLECTION (hashset)) == 0);
	(hashset == NULL ? NULL : (hashset = (g_object_unref (hashset), NULL)));
}


static void test_hashset_iterator (void) {
	GeeHashSet* hashset;
	GeeIterator* iterator;
	GeeIterator* _tmp0;
	char* firstString;
	char* secondString;
	hashset = gee_hash_set_new (G_TYPE_STRING, ((GBoxedCopyFunc) (g_strdup)), g_free, g_str_hash, g_str_equal);
	/* Check iterate empty list*/
	iterator = gee_iterable_iterator (GEE_ITERABLE (hashset));
	g_assert (!gee_iterator_next (iterator));
	/* Check iterate list*/
	gee_collection_add (GEE_COLLECTION (hashset), "42");
	gee_collection_add (GEE_COLLECTION (hashset), "43");
	_tmp0 = NULL;
	iterator = (_tmp0 = gee_iterable_iterator (GEE_ITERABLE (hashset)), (iterator == NULL ? NULL : (iterator = (g_object_unref (iterator), NULL))), _tmp0);
	/* A set is usually not ordered, so this is not a requirement */
	g_assert (gee_iterator_next (iterator));
	firstString = ((char*) (gee_iterator_get (iterator)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), firstString));
	g_assert (gee_iterator_next (iterator));
	secondString = ((char*) (gee_iterator_get (iterator)));
	g_assert (gee_collection_contains (GEE_COLLECTION (hashset), secondString));
	g_assert (!g_str_equal (firstString, secondString));
	/* they can not be identical neither equal*/
	g_assert (!gee_iterator_next (iterator));
	(hashset == NULL ? NULL : (hashset = (g_object_unref (hashset), NULL)));
	(iterator == NULL ? NULL : (iterator = (g_object_unref (iterator), NULL)));
	firstString = (g_free (firstString), NULL);
	secondString = (g_free (secondString), NULL);
}


static void _test_hashset_add_gcallback (void) {
	test_hashset_add ();
}


static void _test_hashset_clear_gcallback (void) {
	test_hashset_clear ();
}


static void _test_hashset_contains_gcallback (void) {
	test_hashset_contains ();
}


static void _test_hashset_remove_gcallback (void) {
	test_hashset_remove ();
}


static void _test_hashset_size_gcallback (void) {
	test_hashset_size ();
}


static void _test_hashset_iterator_gcallback (void) {
	test_hashset_iterator ();
}


static void _main (char** args, int args_length1) {
	g_test_init (&args_length1, &args, NULL);
	g_test_add_func ("/HashSet/Collection/add", _test_hashset_add_gcallback);
	g_test_add_func ("/HashSet/Collection/clear", _test_hashset_clear_gcallback);
	g_test_add_func ("/HashSet/Collection/contains", _test_hashset_contains_gcallback);
	g_test_add_func ("/HashSet/Collection/remove", _test_hashset_remove_gcallback);
	g_test_add_func ("/HashSet/Collection/size", _test_hashset_size_gcallback);
	g_test_add_func ("/HashSet/Iterable/iterator", _test_hashset_iterator_gcallback);
	g_test_run ();
}


int main (int argc, char ** argv) {
	g_type_init ();
	_main (argv, argc);
	return 0;
}




